<?php

/**
 * $Rev: 60 $
 * $Author: omgitsjustjava $
 * $Date: 2016-08-11 11:52:36 +0200 (Do, 11 Aug 2016) $
 */
class PhotoSwipe extends Plugin {

	/**
	 * Flag "laufe unter moziloCMS 2.0"
	 *
	 * @var boolean
	 */
	private $isVersionZweiNull;

	/**
	 * Verzeichnisname des Plugin-Verzeichnisses
	 *
	 * @var string
	 */
	private $PHOTOSWIPE_PLUGINFOLDERNAME = 'PhotoSwipe';

	/**
	 * Verzeichnisname des PhotoSwipe-Unterverzeichnisses
	 *
	 * @var string
	 */
	private $PHOTOSWIPE_SUBFOLDERNAME = 'photoswipe-4.1.1';

	/**
	 * Verzeichnisname des jQuery-Unterverzeichnisses
	 *
	 * @var string
	 */
	private $JQUERY_SUBFOLDERNAME = 'jquery-2.2.2';

	/**
	 * Flag, dass das UI durch eine früher instanziierte Plugin-Instanz bereits initialisiert wurde
	 *
	 * @var boolean
	 */
	private static $uiIsInitialized = false;

	/**
	 * Instanziiert das Plugin-Objekt.
	 */
	function __construct() {
		parent::__construct();
		
		// welche CMS-Version haben wir?
		$this->isVersionZweiNull = (defined('CMSVERSION') && CMSVERSION == '2.0');
	} // __construct()
	
	/**
	 * Gibt den Inhalt des Plugins zurück.
	 *
	 * @param string $value
	 *        	der vom User angegebene Inhalt des Plugin-Platzhalters
	 * @return string der Inhalt des Plugins
	 */
	function getContent($value) {
		global $URL_BASE;
		global $PLUGIN_DIR_NAME;
		global $BASE_DIR;
		global $GALLERIES_DIR_NAME;
		global $PREVIEW_DIR_NAME;
		global $GALLERY_CONF;
		global $specialchars;
		global $CMS_CONF;
		
		// moziloCMS Version 2.0
		if ($this->isVersionZweiNull) {
			$gal_request = $specialchars->replacespecialchars($specialchars->getHtmlEntityDecode($value), false);
			$dir_gallery_src = BASE_DIR . GALLERIES_DIR_NAME . "/" . $gal_request . "/";
			$dir_thumbs = $dir_gallery_src . PREVIEW_DIR_NAME . "/";
			$dir_gallery_url = URL_BASE . GALLERIES_DIR_NAME . "/" . $gal_request . "/";
			$dir_thumbs_url = URL_BASE . GALLERIES_DIR_NAME . "/" . $gal_request . "/" . PREVIEW_DIR_NAME . "/";
		} 		

		// moziloCMS Version 1.12
		else {
			$gal_request = $specialchars->replacespecialchars($specialchars->getHtmlEntityDecode($value), false);
			$dir_gallery_url = $GALLERIES_DIR_NAME . "/" . $gal_request . "/";
			$dir_thumbs_url = $dir_gallery_url . $PREVIEW_DIR_NAME . "/";
			$dir_thumbs_src = $dir_gallery_url . $PREVIEW_DIR_NAME . "/";
			$dir_gallery_src = $BASE_DIR . $GALLERIES_DIR_NAME . "/" . $gal_request . "/";
			
			if ($CMS_CONF->get("modrewrite") == "true") {
				$dir_gallery_url = $URL_BASE . $GALLERIES_DIR_NAME . "/" . $gal_request . "/";
				$dir_thumbs_url = $dir_gallery_url . $PREVIEW_DIR_NAME . "/";
			}
		}
		
		// Galerie gibts nicht?
		if ($value == '' || !file_exists($dir_gallery_src)) {
			return '<span class="deadlink">Galerie ' . $value . ' nicht vorhanden</span>';
		}
		
		$pluginbasepath = $this->isVersionZweiNull ? URL_BASE . PLUGIN_DIR_NAME . '/' : $URL_BASE . $PLUGIN_DIR_NAME . '/';
		$plugindirpath = $pluginbasepath . $this->PHOTOSWIPE_PLUGINFOLDERNAME . '/';
		
		$jsincludes = self::$uiIsInitialized ? '' : '<script type="text/javascript" src="' . $plugindirpath . $this->JQUERY_SUBFOLDERNAME . '/jquery-2.2.2.min.js"></script>
			<script type="text/javascript" src="' . $plugindirpath . $this->PHOTOSWIPE_SUBFOLDERNAME . '/photoswipe.min.js"></script>
			<script type="text/javascript" src="' . $plugindirpath . $this->PHOTOSWIPE_SUBFOLDERNAME . '/photoswipe-ui-default.min.js"></script>
			<script type="text/javascript" src="' . $plugindirpath . 'photoswipe.js"></script>
';
		$cssincludes = self::$uiIsInitialized ? '' : '<link rel="stylesheet" property="stylesheet" href="' . $plugindirpath . $this->PHOTOSWIPE_SUBFOLDERNAME . '/photoswipe.css">
			<link rel="stylesheet" property="stylesheet" href="' . $plugindirpath . $this->PHOTOSWIPE_SUBFOLDERNAME . '/default-skin/default-skin.css">
			<link rel="stylesheet" property="stylesheet" href="' . $plugindirpath . 'plugin.css">
';
		
		// Bilder einlesen
		$imgs = $this->getGalleryImagesAsArray($dir_gallery_src);
		
		// Beschreibungen holen
		$imgdescs = null;
		if ($this->isVersionZweiNull) {
			if (is_file($dir_gallery_src . "/texte.conf.php")) {
				$imgdescs = new Properties($dir_gallery_src . "texte.conf.php");
			}
		} else {
			$imgdescs = new Properties($dir_gallery_src . '/texte.conf');
		}
		
		$gallerycontent = '<div class="picture" itemscope itemtype="http://schema.org/ImageGallery">';
		$picsperrow = $GALLERY_CONF->get("gallerypicsperrow");
		if (($picsperrow == "") || ($picsperrow == 0))
			$picsperrow = 4;
		$gallerycontent .= '<table class="gallerytable"><tr>';
		$i = 0;
		for($i = 0; $i < count($imgs); $i++) {
			$img = $imgs [$i];
			$img = $specialchars->replaceSpecialChars($img, true);
			$desc = isset($imgdescs) ? $imgdescs->get($imgs [$i]) : '&nbsp;';
			$caption = '';
			if (strlen($desc) > 0) {
				$caption = $desc;
				$desc = $specialchars->rebuildSpecialChars($desc, false, true);
			} else {
				$desc = "&nbsp;";
			}
			// Neue Tabellenzeile aller picsperrow Zeichen
			if (($i > 0) && ($i % $picsperrow == 0)) {
				$gallerycontent .= "</tr><tr>";
			}
			$imgsize = getimagesize($dir_gallery_src . $imgs [$i]);
			$thumbsize = getimagesize($dir_gallery_src . $imgs [$i]);
			$gallerycontent .= '<td class="gallerytd" style="width:' . floor(100 / $picsperrow) . '%;">';
			$gallerycontent .= '  <figure itemprop="associatedMedia" itemscope itemtype="http://schema.org/ImageObject" data-index="' . $i . '">
    <a href="' . $specialchars->replaceSpecialChars($dir_gallery_url . $imgs [$i], true) . '" itemprop="contentUrl" data-size="' . $imgsize [0] . 'x' . $imgsize [1] . '" data-index="' . $i . '" title="' . $desc . '">
      <img class="thumbnail" src="' . $specialchars->replaceSpecialChars($dir_thumbs_url . $imgs [$i], true) . '" itemprop="thumbnail" alt="' . $desc . '">
    </a><figcaption>' . $caption . '</figcaption>
  </figure>';
			$gallerycontent .= '</td>';
		}
		// letzte Zeile auffüllen
		while ($i % $picsperrow > 0) {
			$gallerycontent .= '<td class="gallerytd">&nbsp;</td>';
			$i++;
		}
		
		$gallerycontent .= '</tr></table></div>';
		
		$photoswipeoverlay = self::$uiIsInitialized ? '' : '<div class="pswp" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="pswp__bg"></div>
    <div class="pswp__scroll-wrap">
 
        <div class="pswp__container">
            <div class="pswp__item"></div>
            <div class="pswp__item"></div>
            <div class="pswp__item"></div>
        </div>
 
        <div class="pswp__ui pswp__ui--hidden">
            <div class="pswp__top-bar">
                <div class="pswp__counter"></div>
                <button class="pswp__button pswp__button--close" title="Schließen (Esc)"></button>
                <button class="pswp__button pswp__button--share" title="Teilen"></button>
                <button class="pswp__button pswp__button--fs" title="Vollbild-Ansicht"></button>
                <button class="pswp__button pswp__button--zoom" title="Zoomen"></button>
                <div class="pswp__preloader">
                    <div class="pswp__preloader__icn">
                      <div class="pswp__preloader__cut">
                        <div class="pswp__preloader__donut"></div>
                      </div>
                    </div>
                </div>
            </div>
            <div class="pswp__share-modal pswp__share-modal--hidden pswp__single-tap">
                <div class="pswp__share-tooltip"></div> 
            </div>
            <button class="pswp__button pswp__button--arrow--left" title="Voriges Bild (Pfeiltaste links)">
            </button>
            <button class="pswp__button pswp__button--arrow--right" title="Nächstes Bild (Pfeiltaste rechts)">
            </button>
            <div class="pswp__caption">
                <div class="pswp__caption__center"></div>
            </div>
        </div>
    </div>
</div>';
		
		$script = '<script type="text/javascript">initPhotoSwipePlugin();</script>';
		
		self::$uiIsInitialized = true;
		return $jsincludes . $cssincludes . $gallerycontent . $photoswipeoverlay . $script;
	} // function getContent
	
	/**
	 * Gibt die Plugin-Konfiguration als Array zurück.
	 *
	 * @return array die Plugin-Konfiguration
	 */
	function getConfig() {
		// nix zu konfigurieren :)
		$config = array();
		return $config;
	} // getConfig()
	
	/**
	 * Gibt die Plugin-Infos als Array zurück.
	 *
	 * @return array die Plugin-Infos
	 */
	function getInfo() {
		$rev = preg_replace("/[^0-9]/", "", '$Rev: 60 $');
		
		$info = array(
			// Plugin-Name
			"<b>PhotoSwipe-Galerie</b> Revision " . $rev,
			// Plugin-Version
			'1.12 / 2.0',
			// Kurzbeschreibung
			'Die moziloCMS-Implementierung des freien Galerie-Scripts PhotoSwipe (http://photoswipe.com), welches nicht nur eine sehr repräsentative Optik hat, sondern auch Wischgesten auf Mobilgeräten unterstützt.<br />
			<br />
			<span style="font-weight:bold;">Verwendung:</span><br />
			{PhotoSwipe|NameDerGalerie}<br />
			<br />
			<span style="font-weight:bold;">Known Issues:</span><br />
			- AutoPlay funktioniert unter Android nicht<br />
			- Option einbauen: "Mehrere Galerien pro Seite: Einzeln oder alle zusammen anzeigen?"<br />
			- prüfen: Aufruf auch für Einzelbilder sinnvoll/möglich?<br />
			- mögliche Konfigurationsoptionen: AutoPlay-Intervall; Options für die PhotoSwipe-Initialisierung (am besten direkt als JS in einer Textarea; Standard nehmen, wenn leer)<br />
			',
			// Name des Autors
			"Arvid Zimmermann",
			// Download-URL
			"http://www.arvidzimmermann.de",
			array(
				'{PhotoSwipe|}' => 'PhotoSwipe-Galerie'
			)
		);
		
		return $info;
	} // getInfo()
	
	/**
	 * Gibt die Dateinamen der Bilder des Gallerieverzeichnisses mit dem übergeben Pfad als Array zurück.
	 *
	 * @param string $gallerypath
	 *        	der absolute Pfad zum Galerieverzeichnis
	 * @return array die Dateinamen der Bilder
	 */
	function getGalleryImagesAsArray($gallerypath) {
		$imgs = array();
		$galdir = opendir($gallerypath);
		while ($file = readdir($galdir)) {
			if (isValidDirOrFile($file) && $file != 'texte.conf' && $file != 'vorschau') {
				// ... wenn alles passt, ans Bilder-Array anhängen
				$imgs [] = $file;
			}
		}
		closedir($galdir);
		sort($imgs);
		return $imgs;
	}

}